# pio_irq_test.py - 12.08.2021

# Jede der 8 Statemachine führt den gleichen asm_pio-Programmcode aus,
# es wird also dort immer der gleiche irq(rel(0)) aufgerufen.
# Ausgerufen wird von jeder Statemachine ein eigener IRQ-Handler.
# Jede Statemachine erhält ein anderes DatenByte zugewiesen, das innerhalb
# der Statemachine ins ISR kopiert und gepushed wird.
# 
# Damit kann geprüft werden, ob der irq(rel()) für jede Statemachine unabhängig
# und ohne beeinflussung der anderen bzw. durch andere ausgeführt wird.
#
# Das Ergebnis dieses Tests ist leider nicht wie erhofft:
# Nur max. 6 Statemachines arbeiten wie erwartet.


import array as arr
from rp2 import StateMachine, asm_pio
from machine import lightsleep


class IRQ_Raiser(object):
    def __init__(self, statemachine, irq_handler=None, freq=2000):
        self.sm = StateMachine(statemachine, self.irq_raiser, freq=freq)
        if irq_handler is not None:
            self.sm.irq(irq_handler)
        self.sm.active(1)
    
    def put(self, value):
        self.sm.put(value)
        
    def get(self):
        return self.sm.get()
 
    @staticmethod
    @asm_pio()
    def irq_raiser():

        wrap_target()
        pull(block)          #  1 Warten auf ein put()
        mov(isr, osr)        #  2 Eingabe nach isr
        push()               #  3 rausschreiben
        irq(rel(0))          #  4 Interrupt auslösen
        wrap()
    
if __name__ == "__main__":
    
    rx = arr.array("B", [0] * 8)
    
    # Die IRQ-Handler incrementieren lediglich einen Zähler
    # für registrierte Interrupts
    def irq_handler0(_):
        rx[0] += 1

    def irq_handler1(_):
        rx[1] += 1
        
    def irq_handler2(_):
        rx[2] += 1

    def irq_handler3(_):
        rx[3] += 1

    def irq_handler4(_):
        rx[4] += 1

    def irq_handler5(_):
        rx[5] += 1

    def irq_handler6(_):
        rx[6] += 1

    def irq_handler7(_):
        rx[7] += 1


    irq0 = IRQ_Raiser(0, irq_handler=irq_handler0)    
    irq1 = IRQ_Raiser(1, irq_handler=irq_handler1)
    irq2 = IRQ_Raiser(2, irq_handler=irq_handler2)
    irq3 = IRQ_Raiser(3, irq_handler=irq_handler3)
    irq4 = IRQ_Raiser(4, irq_handler=irq_handler4)
    irq5 = IRQ_Raiser(5, irq_handler=irq_handler5)
    irq6 = IRQ_Raiser(6, irq_handler=irq_handler6)
    irq7 = IRQ_Raiser(7, irq_handler=irq_handler7)
    
    for i in range(2):
        irq0.put(i +  0)
        irq1.put(i + 10)
        irq2.put(i + 20)
        irq3.put(i + 30)
        irq4.put(i + 40)
        irq5.put(i + 50)
        irq6.put(i + 60)
        irq7.put(i + 70)
        

    while True:
        
        lightsleep(10)
        # Wenn ein INterrupt registriert wurde, dann den
        # Inhalt aus seinem ISR ausgeben,
        # sollte identisch mit dem Wert sein, der vorher
        # ins SR geschreiben wurde
        
        if rx[0]:
            rx[0] -= 1
            print("irq 0: {:02}".format(irq0.get()))
        
        if rx[1]:
            rx[1] -= 1
            print("irq 1: {:02}".format(irq1.get()))
            
        if rx[2]:
            rx[2] -= 1
            print("irq 2: {:02}".format(irq2.get()))
            
        if rx[3]:
            rx[3] -= 1
            print("irq 3: {:02}".format(irq3.get()))
            
        if rx[4]:
            rx[4] -= 1
            print("irq 4: {:02}".format(irq4.get()))
            
        if rx[5]:
            rx[5] -= 1
            print("irq 5: {:02}".format(irq5.get()))
        
        if rx[6]:
            rx[6] -= 1
            print("irq 6: {:02}".format(irq6.get()))
        
        if rx[7]:
            rx[7] -= 1
            print("irq 7: {:02}".format(irq7.get()))
        
        #break
    lightsleep(100)
    print("\ndone")
    