# pio_taster.py - 12.09.2021
#
# Ein Taster wird benutzt, um einen PIN ein und wieder auszuschalten

# Der Taster regiert nach dem Drücken für einige Zeit nicht.
# Solange der Pin high ist, wird eine innere Schleife mit einem Delay
# von 6ms abgearbeitet.
# Wenn der Pin dann für 25ms low ist, wird die äußere Schleife verlassen. 
# Der Schaltvorgang erfolgt am Anfang der Schleifen.


import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin

@asm_pio(set_init=PIO.OUT_LOW, in_shiftdir=rp2.PIO.SHIFT_LEFT,)

def taster():
    wrap_target()
    
    set(x, 5)                 #   1 Auszeit, in der nicht auf die Taste reagiert wird
    label("wait")
    jmp(pin, "wait")    [10]  #   2
    jmp(x_dec, "wait")   [5]  #   3 20 * 20 * 0.5 ms = 200 ms
                              
    wait(1, pin, 0)           #   4 Warten auf High
    set(pins, 1)              #   5

    set(x, 5)                 #   6 Auszeit, in der nicht auf die Taste reagiert wird
    label("wait1")
    jmp(pin, "wait1")    [10] #   7
    jmp(x_dec, "wait1")   [5] #   8

    wait(1, pin, 0)           #   9 Warten auf High
    set(pins, 0)              #  10
    
    wrap()

class Taster(object):
    def __init__(self, statemachine, input_pin, output_pin):
        self.sm = StateMachine(statemachine, taster,
                            set_base=Pin(output_pin), 
                            in_base=Pin(input_pin, Pin.IN, Pin.PULL_DOWN),
                            jmp_pin=Pin(input_pin, Pin.IN, Pin.PULL_DOWN),
                            freq=2000)
        self.sm.restart()
        self.sm.active(1)

if __name__ == "__main__":
    
    input_pin = 17
    output_pin = 0
    
    t = Taster(statemachine=0, input_pin=input_pin, output_pin=output_pin)      
        
    print("running in statemachine ...\n")
