# pio_uart_tx_osre.py - 12.09.2021
#
# UART Transmitter, ohne Parity-Bit, optional mit 2 Stopbits
# Verwendet keinen Bitzähler, sondern
# - schreibt 8 Bit raus per pull_thresh = 8 und
# - prüft das Byteende mit jmp(not_osre, "label")
# - spart dadurch die Instruktion "set(x, 7)" ein.

from machine import Pin
from rp2 import PIO, StateMachine, asm_pio
import time

@asm_pio(out_init=PIO.OUT_HIGH,
         set_init=PIO.OUT_HIGH,
         fifo_join=rp2.PIO.JOIN_TX,
         out_shiftdir=PIO.SHIFT_RIGHT,
         pull_thresh=8)

def my_uart_tx():

    wrap_target()
    pull()                     #  1 Warten auf Daten 
                               #    Jetzt sind Daten eingegangen ...
    set(pins, 0)           [8] #  2 Das Startbit setzen für 9 Takte
    
    label("next_bit")
    out(pins, 1)           [7] #  3 Das nächste Bit ausgeben

    jmp(not_osre, "next_bit")  #  4 nächstes Bit ausgeben, solange OSR Daten enthält
                               #    Alle 8 Bits sind rausgeshiftet
                               #    1 Stopbit senden
    set(pins, 1)           [7] #  5 Das Stopbit für 9 Takt setzen (1 + 7 + pull )
                               #    alternativ: 2 Stopbits senden
    #set(pins, 1)         [16] #  5 Das Stopbit für 18 Takt setzen (1 + 16 + pull)

    wrap()

# ---Einstelungen --------------------

tx_pin = 16
BAUD = 9600

# ------------------------------------

sm = StateMachine(4, my_uart_tx, freq=9* BAUD,
                  out_base=Pin(tx_pin),
                  set_base=Pin(tx_pin) )
sm.restart()
sm.active(1)

last = time.ticks_us()
for i in range(32):
    sm.put(65 + i)
    
now = time.ticks_us()

print("Time: {} ms".format(time.ticks_diff(now, last)/ 1000))

time.sleep(.5)

print("\ndone")
