# pio_uart_tx_parity_V1.py - 12.09.2021
#
# uart Transmitter mit Parity-bit, optinonal mit 2 Stoppbits
#
# Parity-Bit:
# Am Beginn eines Bytes wird das X-Register mit einem graden Wert (für Even-Parity)
# bzw. einem ungeraden Wert (für Odd-Parity) vorbelegt (der Wert muss größer als 8 sein).
# Nach dem Schreiben eines Bits wird dieses wieder in das ISR eingeelsen
# (nachdem das ISR vorher geleert worden ist).
# Dann wird das ISR ins Y-Register kopiert.
# Dort wird ein Vergleich auf not_Null ausgeführt und
# das X-Register bei Y > 0 decrementiert.
# Am Ende wird das niedrigstwertige Bit des X-Registers als Parity-Bit ausgegeben.
#
# Vorteil: benöitgt keinen zusätzlichen Pin
# NAchtei: benötigt 14 Instruktionen (3 Instruktionen mehr als die andere Variante)

from machine import Pin
from rp2 import PIO, StateMachine, asm_pio
import time

@asm_pio(out_init=PIO.OUT_HIGH,
         set_init=PIO.OUT_HIGH,
         fifo_join=rp2.PIO.JOIN_TX,
         out_shiftdir=PIO.SHIFT_RIGHT,
         pull_thresh=8)

def my_uart_tx():

    wrap_target()
    pull()                     #  1 Warten auf Daten
                               #    X-Register nutzen für Paritätaberechnung
                               #    X darf nicht zu 0 deccrementiert werden,
                               #    daher muss X > 8 sein,
    #set(x, 16)                #  2 Geradzahliges  X -> Paritätsbit = 1 bei Even Parity
    set(x, 15)                 #  2 Ungradzahliges X -> Paritätsbit = 1 bei Odd  Parity
    
                               #    Jetzt sind Daten eingegangen ...
    set(pins, 0)           [8] #  3 Das Startbit setzen für 9 Takte
    
    label("next_bit")
    out(pins, 1)           [2] #  4 Das nächste Bit ausgeben
    mov(isr, null)             #  5 isr zurücksetzen
                               #    1 + 2 + mov + in + mov + jmp + nop + jmp = 9 Takte
    in_(pins, 1)               #  6 Den Bit-WErt einlesen nach isr
    mov(y, isr)                #  7 zum Vergleich ins Y-Register kopieren
    jmp(not_y, "next1")        #  8 wenn y == 0,  dann Nichts tun
    jmp(x_dec, "next")         #  9 wenn y  > o, dann x decrementieren
                               #    x == 0 tritt nie ein, da x groß genug gewählt
    label("next1")
    nop()                      # 10 Ausgleichstakt für jmp(x_dec)
    
    label("next")
    jmp(not_osre, "next_bit")  # 11 nächstes Bit ausgeben, solange OSR Daten enthält
                               #    Alle 8 Bits sind rausgeschiftet
    mov(osr, x)                # 12 x nach OSR kopiern
    out(pins, 1)           [8] # 13 und das niedrigstwertige Bit als Parity-Bit ausgeben
                               #    1 Stopbit senden
    set(pins, 1)           [6] # 14 Das Stopbit für 9 Takt setzen (1 + 6 + pull + set)
                               #    alternativ: 2 Stopbits senden
    #set(pins, 1)         [15] # 14 Das Stopbit für 18 Takt setzen (1 + 15 + pull + set)
    wrap()

# --- Einstellungen ---------------------------

tx_pin = 16
BAUD = 9600

# ---------------------------------------------

sm = StateMachine(4, my_uart_tx, freq=9* BAUD,
                  out_base=Pin(tx_pin),
                  set_base=Pin(tx_pin),
                  in_base=Pin(tx_pin) )
sm.restart()
sm.active(1)

time.sleep(.1)
for i in range(4):
    sm.put(0x55 + i)

time.sleep(.5)

print("\ndone")
